/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*!
   \file visor.h
   \brief include file for visor cgi binary
*/

// system include files
#include <iostream>
#include <vector>
#include <string>
#include <cstring>
#include <exception>
#include "errno.h"

// cgicc include files
#include "cgicc/Cgicc.h"
#include "cgicc/HTTPHTMLHeader.h"
#include "cgicc/HTMLClasses.h"

// includes from project
#include "metdb.h"
#include "metcal.h"
#include "wwsr.h"
#include "metgra.h"

// masks about quality/existence of data in a met_report 
#define MMASK_HUM_INT 1
#define MMASK_HUM_EXT 2
#define MMASK_TEMP_INT 4
#define MMASK_TEMP_EXT 8
#define MMASK_PRES 16
#define MMASK_WSPD 32
#define MMASK_WDIR 64
#define MMASK_RAIN 128
#define MMASK_INTERV 256
#define MMASK_DATIME 512
#define MMASK_PREC MMASK_RAIN
#define MMASK_WWSR 1023

/*! 
  \def  MMASK_FAKERAIN 
  Sometimes a fake pulse from pluviometer is acumulated. This is
  to mark the possible pulse as false
*/ 
#define MMASK_FAKERAIN 1024

using namespace std;
using namespace cgicc;

class met_station {
   public:
   int mask;
   string confile; 
   string nocal;
   struct station_conf conf;
   struct station_cal cal;
   struct station_cal *calp;
   DB *db;

   void set_mask (int m);

   met_station (int m);

   met_station ();

   int get_station_conf(void);

   int open_met_db(void);

   int close_met_db(void);

   int get_station_cal(void);
};


class met_report {
  public:
    int mask;
    time_t datime;
    int interval;
    double temp_int;
    double temp_out;
    double hum_int;
    double hum_out;
    double tdew_int;
    double tdew_out;
    double press;
    double sea_press;
    double wspd;
    double wdir;
    double wgust;
    double wdgust;
    double rain;

    // functions
    met_report();

    met_report(struct real_data& r);

    met_report(struct obs_data *o, met_station& e);

    //copy constructor
    met_report (const met_report &r);

    // overload of =
    met_report& operator= (const met_report &r);

};

class var_sum {
  public:
  size_t n; /*!< Number of data*/
  double tiempo; /*!< Amount of seconds with data */
  double vmax;  /*!< Max value */
  time_t tvmax; /*!< unix time of max valur */
  double vmin;  /*!< min value */
  time_t tvmin;  /*!< unix time of min value */
  double acum; /*!< Amount / mean of data */

  var_sum()
  {
     memset(this, 0, sizeof(class var_sum));
  }

};

// for non-symetric vars as prec or wind
class var_sum_nosim {
  public:
  size_t n; /*!< Number of data*/
  double tiempo; /*!< Amount of seconds with data */
  double vmax;  /*!< Max value */
  time_t tvmax; /*!< unix time of max valur */
  double aux;  /*!< Max value */
  time_t taux; /*!< unix time of max valur */
  double acum; /*!< Amount / mean of data */

  var_sum_nosim()
  {
     memset(this, 0, sizeof(class var_sum_nosim));
  }

   
};

class met_stat {
  public:
  int mask; /*!< mask to summaize what parameters have valid data */
  size_t n; /*!< Number of observations (array of simultaneous parameters values) are included */
  time_t begin; /*!< Unix time of the begining of interval */
  time_t end; /*!< Unix time for the end of interval */
  time_t latest; /*!< Unix time of latest sumarized data */
  var_sum hum_int; /*!< stat resul for interior humidity */
  var_sum temp_int; /*!< stat resul for interior temperature */
  var_sum tdew_int; /*!< stat resul for interior dewpoint temperature */
  var_sum hum_out; /*!< stat resul for exterior humidity */
  var_sum temp_out; /*!< stat resul for exterior temperature */
  var_sum tdew_out; /*!< stat resul for exterior dewpoint temperature */
  var_sum sea_press; /*!< stat resul for pressure at station level */
  var_sum press; /*!< stat resul for pressure at station level */
  var_sum_nosim wspd; /*!< stat resul for wind speed */
  var_sum wdir; /*!< stat resul for wind direction */
  var_sum_nosim wgust; /*!< stat resul for wind speed */
  var_sum_nosim rain; /*!< stat resul for accumulated precipitation */

  met_stat()
  {
     memset(this, 0, sizeof(class met_stat));
  }

  void clean()
  {
     memset(this, 0, sizeof(class met_stat));
  }
};

#define MAX_ARGLEN 64

#define NUMOBS 4096

#define NGARGS 9

int read_getargs(Cgicc& cgi, met_station& s);
int read_arg(string& arg, Cgicc& cgi, const char *name);
int show_met_report_txt(met_report& r, met_station& e);
int interp_met_report(met_report& r, struct obs_data *o, size_t nobs, time_t datime,
                met_station& e);
int summarize_met_report(met_stat& s, const met_report& r);
int print_met_stat(met_stat& S, int viewmask);
char *dirwtxt(char *cdir, double wdir);
int print_info_station(met_station& e);
int get_summary_data(met_stat& s, time_t tbegin, time_t tend, met_station& e);
int get_summary_data(met_report *d, size_t numobs, met_stat& s, time_t tbegin, time_t tend, met_station& e);
int get_data(met_report **d, time_t tbegin, time_t tend, met_station& e);
int print_tableres(met_station& e, int tbegin, int nd, int viewmask);
int print_header_metstat(int viewmask);
int print_row_metstat(met_stat& S, int viewmask);

void met2metgra(metgra_report& g, met_report& d);
int prepara_metgra(metgra& g, met_station& e, met_stat& s, time_t tbegin, int ndays);
int plot_temp_ext(metgra& g, met_stat &s);
int plot_temp_int(metgra& g, met_stat &s);
int data_plot(metgra& g, met_station& e, met_stat &s, time_t tref, int ndays, int mask);
int print_prolog(met_station& e);
int print_epilog();
int print_menu();
int print_css();


extern string Fecha;
extern int SHOWLAST, NOW, ND, NDG, GTASK_MASK;
extern time_t TBEGIN, TEND;
extern char REMOTEHOST[64];
extern char GARGS[NGARGS][8];
extern int GARGS_M[NGARGS];
extern char GARGS_NAME[NGARGS][24];

