/* $Id$ */ 
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "metgra.h"


// definiciones particulares de esta rutina
#define degs RAD2GRA
#define rads GRA2RAD
#define pi M_PI


double SunDia = 0.53;     // Sunradius degrees

double AirRefr = 34.0/60.0; // athmospheric refraction degrees //

//   Get the days to J2000
//   h is UT in decimal hours
//   FNday only works between 1901 to 2099 - see Meeus chapter 7

double FNday (int y, int m, int d, float h)
{
  long int luku = - 7 * (y + (m + 9)/12)/4 + 275*m/9 + d;
  // type casting necessary on PC DOS and TClite to avoid overflow
  luku += (long int)y * 367;
  return (double)luku - 730531.5 + h/24.0;
};

//   the function below returns an angle in the range
//   0 to 2*pi

double FNrange (double x)
{
  double b = 0.5 * x / pi;
  double a = 2.0 * pi * (b - (long)(b));
  if (a < 0)
    a = 2.0 * pi + a;
  return a;
};

// Calculating the hourangle
//
double f0(double lat, double declin)
{
  double fo, dfo;
  // Correction: different sign at S HS
  dfo = rads * (0.5 * SunDia + AirRefr);
  if (lat < 0.0)
    dfo = -dfo;
  fo = tan(declin + dfo) * tan(lat * rads);
  if (fo > 0.99999)
    fo = 1.0; // to avoid overflow //
  if (fo < -0.99999)
    fo = -1.0; // to avoid overflow //
  fo = asin(fo) + pi / 2.0;
  return fo;
};

// Calculating the hourangle for twilight times
//
double f1(double lat, double declin)
{
  double fi, df1;
  // Correction: different sign at S HS
  df1 = rads * 6.0;
  if (lat < 0.0)
    df1 = -df1;
  fi = tan(declin + df1) * tan(lat * rads);
  if (fi > 0.99999)
    fi = 1.0; // to avoid overflow //
  if (fi < -0.99999)
    fi = -1.0; // to avoid overflow //
  fi = asin(fi) + pi / 2.0;
  return fi;
};


//   Find the ecliptic longitude of the Sun

double FNsun (double d, struct astro *a)
{
  double g;
  //   mean longitude of the Sun

  a->L = FNrange(280.461 * rads + .9856474 * rads * d);

  //   mean anomaly of the Sun

  g = FNrange(357.528 * rads + .9856003 * rads * d);

  //   Ecliptic longitude of the Sun

  return FNrange(a->L + 1.915 * rads * sin(g) + .02 * rads * sin(2 * g));
};

/* Nueva version mas aproximada */
double FNsun2 (double d, struct astro *a)
{
  double w, M, v, r, g;
  double ecc, E, lonsun, yequat, zequat;

  //   mean longitude of the Sun
  w = 282.9404 + 4.70935E-5 * d;
  M = 356.047 + 0.9856002585 * d;

  // Sun's mean longitude
  a->L = FNrange(w * rads + M * rads);

  //   mean anomaly of the Sun

  g = FNrange(M * rads);

  // eccentricity
  ecc = 0.016709 - 1.151E-9 * d;

  //   Obliquity of the ecliptic

  a->obliq = 23.4393 * rads - 3.563E-7 * rads * d;
  E = M + degs * ecc * sin(g) * (1.0 + ecc * cos(g));
  E = degs * FNrange(E * rads);
  a->x = cos(E * rads) - ecc;
  a->y = sin(E * rads) * sqrt(1.0 - ecc * ecc);
  r = sqrt(a->x * a->x + a->y * a->y);
  v = atan2(a->y, a->x) * degs;

  // longitude of sun
  lonsun = v + w;
  if (lonsun>360.0)
    lonsun-= 360.0;

  // sun's ecliptic rectangular coordinates
  a->x = r * cos(lonsun * rads);
  a->y = r * sin(lonsun * rads);
  yequat = a->y * cos(a->obliq);
  zequat = a->y * sin(a->obliq);
  a->RA = atan2(yequat, a->x);
  a->delta = atan2(zequat, sqrt(a->x * a->x + yequat * yequat));
  a->RA *= degs;

  //   Ecliptic longitude of the Sun

  return FNrange(a->L + 1.915 * rads * sin(g) + .02 * rads * sin(2 * g));
};


// Display decimal hours in hours and minutes
void showhrmn(double dhr)
{
  int hr, mn;
  hr = (int) dhr;
  mn = (dhr - (double) hr) * 60;

  printf("%0d:%0d",hr,mn);
};

/*!
   \fn int metgra_sunpos(double *azim, double *altit, struct tm *hora, struct astro *a, double lat, double lon, int mode)
 
   \brief da la altura y azimut a una hora y posicion geografica
   \param azim puntero donde se coloca el azimut del sol, en grados
   \param altit puntero donde se coloca la altura del sol, en grados
   \param hora puntero a struct tm donde esta la hora UTC en la que se realizan los calculos
   \param a puntero a struct \ref astro donde se guardan coordenadas
   \param lat latitud del lugar, en grados
   \param lon longitude del lugar, en grados 
   \param mode modo de operacion. Si mode = 1; se supone que los elemetos dependientes de la hora
     ya han sido y colocados en variables globales 
*/
int metgra_sunpos(double *azim, double *altit, struct tm *hora, struct astro *a, double lat, double lon, int mode)
{
  int year, m, day;
  double d, lambda;
  double sidtime, ha, xhor, yhor, zhor;

  if (mode == 0)
    {
      year = hora->tm_year;
      year += 1900;
      m = hora->tm_mon + 1;
      day = hora->tm_mday;

      a->h = (double)hora->tm_hour + ((double)hora->tm_min)/60.0 + ((double)hora->tm_sec)/3600.0;

      d = FNday (year, m, day, a->h);

      //   Use FNsun2 to find the ecliptic longitude of the
      //   Sun

      lambda = FNsun2(d, a);

      //   Find the RA and DEC of the Sun

      /*alpha = atan2(cos(obliq) * sin(lambda), cos(lambda));*/
      /*delta = asin(sin(obliq) * sin(lambda));*/

      a->gmst0 = a->L  * degs / 15.0 + 12.0;
    }
  sidtime = a->gmst0 + a->h + lon / 15.0;
  ha = 15.0 * sidtime - a->RA;
  ha = FNrange(rads * ha);
  a->x = cos(ha) * cos(a->delta);
  a->y = sin(ha) * cos(a->delta);
  a->z = sin(a->delta);

  xhor = a->x * sin(lat * rads) - a->z * cos(lat * rads);
  yhor = a->y;
  zhor = a->x * cos(lat * rads) + a->z * sin(lat * rads);
  *azim = atan2(yhor, xhor) + pi;
  *azim = FNrange( *azim ) * degs;
  *altit = asin(zhor) * degs;

  // Include Air refraction if altitude less than 30 degrees
  if (*altit < 30.0)
    *altit += AirRefr;
  /*printf("alt=%.1lf azi=%.1lf<br>\n", *altit, *azim);*/
  return 1;
}

int metgra_ortocaso(double latit, double longit, struct tm *hora0, struct astro *a)
{
  double azim, alt;
  metgra_sunpos(&azim, &alt, hora0, a, latit, longit, 1);

  if (alt < -6.0)
    {
      return 2;
    }
  else if (alt < 0.0)
    {
      return 1;
    }
  return 0;
}

/* se diferencia de ogimet_ortocaso por el modo de llamar a ogimet_sunpos() */
int metgra_ortocaso2(double latit, double longit, struct tm *hora0, struct astro *a)
{
  double azim, alt;
  metgra_sunpos(&azim, &alt, hora0, a, latit, longit, 0);

  if (alt < -6.0)
    {
      return 2; // night
    }
  else if (alt < 0.0)
    {
      return 1; // twilight
    }
  return 0; // day
}


