/* $Id: wwsr.h 99 2009-06-11 20:55:03Z gbv $ */
/***************************************************************************
 *                                                                         *
 *   wwsr - Wireless Weather Station Reader                                *
 *   2007 dec 19, Michael Pendec (michael.pendec@gmail.com)                *
 *   Version 0.1                                                           *
 *   2008 jan 24 Svend Skafte (svend@skafte.net)                           *
 *   Modifications for different firmware version(?)                       *
 *   2009 Jan 21 Guillermo Ballester gbv@oxixares.com                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*! \file wwsr.h
    \brief include file for wwsr
*/
#ifndef WWSR_H
#define WWSR_H

#ifdef __cplusplus
extern "C"
  {
#endif

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

    /* To active Berkeley DB support */
    /*#define WITH_DB*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <signal.h>
#include <stdint.h>
#include <ctype.h>
#include <time.h>
#include <usb.h>
#include <libintl.h>
#include <errno.h>
#include <math.h>
#ifdef WITH_DB
# include "metdb.h"
#endif

    /* library for correction */
#include "metcal.h"

    /* Masks to select parameters */
#define MASK_HUM_INT 1
#define MASK_HUM_EXT 2
#define MASK_TEMP_INT 4
#define MASK_TEMP_EXT 8
#define MASK_PRES 16
#define MASK_WSPD 32
#define MASK_WDIR 64
#define MASK_RAIN 128
#define MASK_INTERV 256
#define MASK_DATIME 512
#define MASK_WIND (MASK_WSPD | MASK_WDIR)
#define MASK_HUM (MASK_HUM_INT | MASK_HUM_EXT)
#define MASK_TEMP (MASK_TEMP_INT | MASK_TEMP_EXT)
#define MASK_ALL (MASK_HUM | MASK_TEMP | MASK_PRES | MASK_WIND | MASK_RAIN | MASK_INTERV)
#define MASK_PREC MASK_RAIN
    
#define MASK_FAKERAIN 1024

#define PATH_LEN  256

#define MASK_WWSR_NODATA 0xFF
#define MASK_WWSR_FAKERAIN 0x01


    /* size of station memory */
#define MEMSIZE 0x10000

#ifndef GRAD2RAD
# define GRAD2RAD (M_PI / 180.0)
#endif
#ifndef RAD2GRAD
# define RAD2GRAD (180.0 * M_1_PI)
#endif

    /*!
      \struct obs_data
      \brief struct to read a data sample and store it in integer format
    */
    struct obs_data
      {
        time_t datime; /*!< unix time of observation (round to minute) */
        int qflag; /*!< integer with bit mask quality flags. 0 = Good, 1= bad */
        uint16_t interval; /*!< minutes since prior data */
        uint16_t hum_int; /*!< internal humidity in % */
        int16_t temp_int; /*!< Interior temperature in thents of celsius */
        int16_t hum_out; /*!< external humidity in % */
        int16_t temp_out; /*!< external temperature in thents of celsius */
        uint16_t press; /*!< pressure in thenths of Hectopascal */
        uint16_t wspd; /*!< wind speed in thents of m/s */
        uint16_t wgust; /*!< wind gust in thents of m/s */
        uint16_t wdir; /*!< windir in directions */
        uint16_t prec; /*!< accumulated precipitation in 0'3 mm units */
        uint8_t mask; /*!< it is FF if no data is present */
        uint8_t buf[16]; /*!< the buffer src from wwsr */
      };

    /*!
      \struct real_data
      \brief struct to store calibrated data in float form
    */
    struct real_data
      {
        int mask; /*!< bit mask to ressume what valid information the struct have */
        time_t datime; /*!< unix time of observation (round to minute) */
        double interval; /*!< seconds since prior data */
        double hum_int; /*!< internal humidity in % */
        double temp_int; /*!< Interior temperature (celsius) */
        double tdew_int; /*!< Interior dewpoint temperature (celsius) */
        double hum_out; /*!< external humidity in % */
        double temp_out; /*!< external temperature (celsius), computed */
        double tdew_out; /*!< external dewpoint temperature (celsius), computed */
        double pres0; /*!< pressure Hectopascal at station */
        double press; /*!< pressure Hectopascal at sea level*/
        double wspd; /*!< wind speed (km/h) */
        double wgust; /*!< wind gust (km/h) */
        double wdir; /*!< windir in directions */
        double prec; /*!< accumulated precipitation (mm) */
      };

    /*!
      \struct stat_var
      \brief This struct stores the basic statistic data of a parameter
    */
    struct stat_var
      {
        size_t n; /*!< Number of data*/
        size_t tiempo; /*!< Amount of seconds with data */
        double vmax;  /*!< Max value */
        time_t tvmax; /*!< unix time of max valur */
        double vmin;  /*!< min value */
        time_t tvmin;  /*!< unix time of min value */
        double acum; /*!< Amount / mean of data */
      };

    /*!
      \struct stat_summary
      \brief This struct collects all statistic data of a parameter collection during a time interval
    */
    struct stat_summary
      {
        int mask; /*!< mask to summaize what parameters have valid data */
        size_t n; /*!< Number of observations (array of simultaneous parameters values) are included */
        time_t begin; /*!< Unix time of the begining of interval */
        time_t end; /*!< Unix time for the end of interval */
        time_t latest; /*!< Unix time of latest sumarized data */
        struct stat_var hum_int; /*!< stat resul for interior humidity */
        struct stat_var temp_int; /*!< stat resul for interior temperature */
        struct stat_var tdew_int; /*!< stat resul for interior dewpoint temperature */
        struct stat_var hum_out; /*!< stat resul for exterior humidity */
        struct stat_var temp_out; /*!< stat resul for exterior temperature */
        struct stat_var tdew_out; /*!< stat resul for exterior dewpoint temperature */
        struct stat_var pres0; /*!< stat resul for pressure at station level */
        struct stat_var press; /*!< stat resul for pressure at station level */
        struct stat_var wspd; /*!< stat resul for wind speed */
        struct stat_var wdir; /*!< stat resul for wind direction */
        struct stat_var wgust; /*!< stat resul for wind speed */
        struct stat_var prec; /*!< stat resul for accumulated precipitation */
      };

    /*!
      \struct extrem_data
      \brief Sruct to store some exterm event
    */
    struct extrem_data
      {
        double dat; /*!< value of data */
        struct tm tim; /*!< time of the event */
      };


    /*!
      \struct station_conf
      \brief contain station configuration data
    */
    struct station_conf
    {
      char confile[PATH_LEN]; /*!< the path of configuration file */
      char dbfile[PATH_LEN]; /*!< the path of database file */
      char calfile[PATH_LEN]; /*!< the path of calibration file */
      char index[64]; /*!< station index */
      char name[128]; /*!< Common name string */
      double altitude; /*!< Altitude (meters) */
      double latitude; /*!< Latitude (degrees), North positive */
      double longitude; /*!< Longitude (degrees), East positive */
    };

    /************************************/
    int buf2obs (struct obs_data *o, unsigned char *buf);

    uint16_t read_uint16(uint8_t *bf);
    int16_t read_sint16(uint8_t *bf);
    void print_bytes(char *bytes, int len);
    int16_t read_sint16(uint8_t *bf);
    void print_bytes(char *bytes, int len);
    uint8_t bcdval( uint8_t a);
    unsigned int get_pos(uint16_t current, int nobs);
    double value( double x, struct cal_pol *c);

    void print_obs_verbose(FILE *f, struct obs_data *o, time_t tbegin, time_t tend, int viewmask);
    void print_obs_nores(FILE *f, struct obs_data *o, char *sep, int viewmask);
    void print_real_data(FILE *f, struct real_data *r,const char *sep);
    void print_real_data_csv(FILE *f, struct real_data *r, const char *sep);
    void read_date(struct tm *tim, uint8_t *bf);
    void read_extremes(uint8_t *bf);
    int summarize_realdata(struct stat_summary *s, const struct real_data *r);
    void print_stat_summary(struct stat_summary *s, FILE *f);
    int print_stat_summary_csv(struct stat_summary *s, FILE *f, const char *sep, int viewmask);
    void print_all_extremes(FILE *f, int viewmask);
    int obsdata_to_realdata(struct real_data *r, struct obs_data *o, struct station_cal *s, double altitude);
    int interp_data(struct real_data *r, struct obs_data *o, size_t nobs, time_t datime,
                    struct station_cal *s, double altitude);
    int read_station_conf(struct station_conf *e, const char *filename);


#ifdef WITH_DB
    int del_db_obs(DB *dbp, uint8_t mask, time_t tbegin, time_t tend);
    size_t count_db_obs(DB *dbp, time_t tbegin, time_t tend);
    size_t count_db_obs_wide(DB *dbp, time_t tbegin, time_t tend);
    int get_db_obs(DB *dbp, struct obs_data *obs, size_t maxobs, time_t tbegin, time_t tend);
    int get_db_obs_wide(DB *dbp, struct obs_data *obs, size_t maxobs, time_t tbegin, time_t tend);
    int insert_db_obs(DB *dbp, struct obs_data *obs, int overwrite);
    int get_db_obs_last(DB *dbp, struct obs_data *obs, int mode);

#endif

#ifdef __cplusplus
  } /* closing brace for extern "C" */
#endif


#endif /* WWSR_H */

