/* $Id: wwsr_db.c 90 2009-05-31 22:00:39Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*!
  \file wwsr_db.c
  \brief file with support for Berkeley DB
*/
#include "wwsr.h"

#ifdef WITH_DB

/*!
  \fn int insert_db_obs(DB *dbp, struct obs_data *obs)
  \brief Try to insert a record to database
  \param dbp DB pointer already open by caller
  \param obs pointer to a struct \ref obs_data with the data to insert
  \param overwrite
   If the data is rejected a 0 is returned. If inserted then it returns 1.
   If error then returns -1 
*/
int insert_db_obs(DB *dbp, struct obs_data *obs, int overwrite)
{
  int ret, nobs, i;
  uint32_t flag;
  DBT key, data;
  char aux[16];
  struct obs_data obx[3];
  struct tm tim;

  /* check if a valid observadion */
  if ( obs->buf[15] == 0xFF || (obs->buf[0] == 0 && obs->buf[1] == 0))
    {
      return 0;
    }

  /* clear key and data */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  gmtime_r(&obs->datime, &tim);
  strftime(aux,16, "%Y%m%d%H%M", &tim);

  key.data = aux;
  key.size = strlen(aux) + 1;
  data.data = obs->buf;
  data.size = 16;

  if (overwrite)
    {
      flag = 0;
    }
  else
    flag = DB_NOOVERWRITE;

  if ((obs->mask & 0x01) == 0)
    {
      /*
        Check if there are bad overlapped time for non-current observations
        Two observations not marked as current (mask == 0x01) cannot be spaced less than
        two minutes
      */
      nobs = get_db_obs(dbp, &obx[0], 3, obs->datime - 60, obs->datime + 60);
      if (nobs)
        {
          for (i = 0; i < nobs; i++)
            {
              if ((obx[i].mask & 0x01) == 0)
                {
                  /* found overlap, reject */
                  return 0;
                }
            }
        }
    }

  ret = put_record_db(&key, &data, dbp, flag);
  if (ret == 0)
    return 1;
  else if (ret == DB_KEYEXIST)
    return 0;
  else
    return -1;
}

/*!
  \fn int get_db_obs(DB *dbp, struct obs_data *obs, size_t maxobs, time_t tbegin, time_t tend)
  \brief get the data from a time interval.
  \param dbp DB conection pointer already opened by caller
  \param obs array of struct \ref obs_data where to set the data readed from DB
  \param maxobs number of structs \ref obs_data with available memory allocated by caller
  \param tbegin Unix time of the time interval begin
  \param tend Unix time of the end.

  This routines set the data with time equal or greater to \a tbegin and lesser or equal  
  to \a tend . It returns the number of data stored with success.
*/
int get_db_obs(DB *dbp, struct obs_data *obs, size_t maxobs, time_t tbegin, time_t tend)
{
  int no, ret;
  char datei[16], datef[16];
  uint32_t flag;
  time_t ti;
  struct tm tim;
  DBC *curs;
  DBT key, data;

  /* set a cursor */
  dbp->cursor( dbp, NULL, &curs, 0);

  /* set first and last key from the interval*/
  memset(&tim, 0, sizeof(struct tm));
  ti = tbegin;
  gmtime_r(&ti, &tim);
  strftime(datei, 16, "%Y%m%d%H%M", &tim);
  ti = tend;
  gmtime_r(&ti, &tim);
  strftime(datef, 16, "%Y%m%d%H%M", &tim);

  /* set initial key, search */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  key.data = datei;
  key.size = strlen(datei) + 1;

  /* loop to search data */
  no = 0;
  flag = DB_SET_RANGE; /* flag for fisrt time */
  while ((ret = curs->c_get(curs, &key, &data, flag)) == 0 && no < maxobs)
    {
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M", &tim);
      ti = timegm(&tim);
      if (tend == 0 || ti <= tend)
        {
          buf2obs( &obs[no], data.data);
          obs[no].datime = ti;
          no++;
        }
      else
        {
          /* we surpassed the end of interval */
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
    }

  /* close the cursor in case of end or max memory*/
  if (curs != NULL)
    curs->c_close(curs);
  return no;
}

/*!
  \fn size_t count_db_obs(DB *dbp, time_t tbegin, time_t tend)
  \brief count the rows of data in the given time interval.
  \param dbp DB conection pointer already opened by caller
  \param tbegin Unix time of the time interval begin
  \param tend Unix time of the end.

  This routines count the data with time equal or greater to \a tbegin and lesser or equal  
  to \a tend . 
*/
size_t count_db_obs(DB *dbp, time_t tbegin, time_t tend)
{
  size_t no;
  int ret;
  char datei[16], datef[16];
  uint32_t flag;
  time_t ti;
  struct tm tim;
  DBC *curs;
  DBT key, data;

  /* set a cursor */
  dbp->cursor( dbp, NULL, &curs, 0);

  /* set first and last key from the interval*/
  memset(&tim, 0, sizeof(struct tm));
  ti = tbegin;
  gmtime_r(&ti, &tim);
  strftime(datei, 16, "%Y%m%d%H%M", &tim);
  ti = tend;
  gmtime_r(&ti, &tim);
  strftime(datef, 16, "%Y%m%d%H%M", &tim);

  /* set initial key, search */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  key.data = datei;
  key.size = strlen(datei) + 1;

  /* loop to search data */
  no = 0;
  flag = DB_SET_RANGE; /* flag for fisrt time */
  while ((ret = curs->c_get(curs, &key, &data, flag)) == 0)
    {
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M", &tim);
      ti = timegm(&tim);
      if (tend == 0 || ti <= tend)
        {
          no++;
        }
      else
        {
          /* we surpassed the end of interval */
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
    }

  /* close the cursor in case of end or max memory*/
  if (curs != NULL)
    curs->c_close(curs);
  return no;
}


/*!
  \fn int get_db_obs_wide(DB *dbp, struct obs_data *obs, size_t maxobs, time_t tbegin, time_t tend)
  \brief get the data from a time interval that close the solicited time interval
  \param dbp DB conection pointer already opened by caller
  \param obs array of struct \ref obs_data where to set the data readed from DB
  \param maxobs number of structs \ref obs_data with available memory allocated by caller
  \param tbegin Unix time of the time interval begin
  \param tend Unix time of the end.

  The diff with \ref get_db_obs is that if there is no data with the time equal to \a tbegin then it also get the 
  last obs data prior \a tbegin. In the same manner, if no data with time equal to \a tend, it also stores the next obs.
  It returns the number of data stored with success.
*/
int get_db_obs_wide(DB *dbp, struct obs_data *obs, size_t maxobs, time_t tbegin, time_t tend)
{
  int no, ret;
  char datei[16], datef[16];
  uint32_t flag;
  time_t ti;
  struct tm tim;
  DBC *curs;
  DBT key, data;

  /* set a cursor */
  dbp->cursor( dbp, NULL, &curs, 0);

  /* set first and last key from the interval*/
  memset(&tim, 0, sizeof(struct tm));
  ti = tbegin;
  gmtime_r(&ti, &tim);
  strftime(datei, 16, "%Y%m%d%H%M", &tim);
  ti = tend;
  gmtime_r(&ti, &tim);
  strftime(datef, 16, "%Y%m%d%H%M", &tim);

  /* set initial key, search */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  key.data = datei;
  key.size = strlen(datei) + 1;

  /* loop to search data */
  no = 0;
  flag = DB_SET_RANGE; /* flag for fisrt time */

  if ((ret = curs->c_get(curs, &key, &data, flag)) == 0)
    {
      // first valor equal or after datei
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M", &tim);
      ti = timegm(&tim);
      if (ti == tbegin)
        {
          // this record is valid
          buf2obs( &obs[no], data.data);
          obs[no].datime = ti;
          no++;
          flag = DB_NEXT;
        }
      else
        flag = DB_PREV; // we first will load the previous
    }
  else
    {
      if (curs != NULL)
        curs->c_close(curs);
      return no;
    }

  /* First try to read a data */
  if ((ret = curs->c_get(curs, &key, &data, flag)) == 0)
  {
    memset(&tim, 0, sizeof(struct tm));
    strptime( key.data, "%Y%m%d%H%M", &tim);
    ti = timegm(&tim);
      if (tend == 0 || ti <= tend)
        {
          buf2obs( &obs[no], data.data);
          obs[no].datime = ti;
          no++;
        }
      else
        {
          /* we surpassed the end of interval */
          if (obs[no - 1].datime < tend)
          {   
            buf2obs( &obs[no], data.data);
            obs[no].datime = ti;
            no++;
          }
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
  }
  else if (ret == DB_NOTFOUND)
  {
     flag = DB_NEXT;
  }

  /* And now a loop for the next records */
  while ((ret = curs->c_get(curs, &key, &data, flag)) == 0 && no < maxobs)
    {
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M", &tim);
      ti = timegm(&tim);
      if (tend == 0 || ti <= tend)
        {
          buf2obs( &obs[no], data.data);
          obs[no].datime = ti;
          no++;
        }
      else
        {
          /* we surpassed the end of interval */
          if (obs[no - 1].datime < tend)
          {   
            buf2obs( &obs[no], data.data);
            obs[no].datime = ti;
            no++;
          }
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
    }

  /* close the cursor in case of end or max memory*/
  if (curs != NULL)
    curs->c_close(curs);
  return no;
}

/*!
  \fn size_t count_db_obs_wide(DB *dbp, time_t tbegin, time_t tend)
  \brief count the rows of data in a time interval that close the solicited time interval
  \param dbp DB conection pointer already opened by caller
  \param tbegin Unix time of the time interval begin
  \param tend Unix time of the end.

  The diff with \ref get_db_obs is that if there is no data with the time equal to \a tbegin then it also get the 
  last obs data prior \a tbegin. In the same manner, if no data with time equal to \a tend, it also stores the next obs.
*/
size_t count_db_obs_wide(DB *dbp, time_t tbegin, time_t tend)
{
  size_t no;
  int ret;
  char datei[16], datef[16];
  uint32_t flag;
  time_t ti;
  struct tm tim;
  DBC *curs;
  DBT key, data;

  /* set a cursor */
  dbp->cursor( dbp, NULL, &curs, 0);

  /* set first and last key from the interval*/
  memset(&tim, 0, sizeof(struct tm));
  ti = tbegin;
  gmtime_r(&ti, &tim);
  strftime(datei, 16, "%Y%m%d%H%M", &tim);
  ti = tend;
  gmtime_r(&ti, &tim);
  strftime(datef, 16, "%Y%m%d%H%M", &tim);

  /* set initial key, search */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  key.data = datei;
  key.size = strlen(datei) + 1;

  /* loop to search data */
  no = 0;
  flag = DB_SET_RANGE; /* flag for fisrt time */

  if ((ret = curs->c_get(curs, &key, &data, flag)) == 0)
    {
      // first valor equal or after datei
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M", &tim);
      ti = timegm(&tim);
      if (ti == tbegin)
        {
          // this record is valid
          no++;
          flag = DB_NEXT;
        }
      else
        flag = DB_PREV; // we first will load the previous
    }
  else
    {
      if (curs != NULL)
        curs->c_close(curs);
      return no;
    }

  /* First try to read a data */
  if ((ret = curs->c_get(curs, &key, &data, flag)) == 0)
  {
    memset(&tim, 0, sizeof(struct tm));
    strptime( key.data, "%Y%m%d%H%M", &tim);
    ti = timegm(&tim);
      if (tend == 0 || ti < tend)
        {
          no++;
        }
      else
        {
          /* we surpassed the end of interval */
          if (no)
          {   
            no++;
          }
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
  }
  else if (ret == DB_NOTFOUND)
  {
     flag = DB_NEXT;
  }

  /* And now a loop for the next records */
  while ((ret = curs->c_get(curs, &key, &data, flag)) == 0)
    {
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M", &tim);
      ti = timegm(&tim);
      if (tend == 0 || ti < tend)
        {
          no++;
        }
      else
        {
          /* we surpassed the end of interval */
          if (no)
          {   
            no++;
          }
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
    }

  /* close the cursor in case of end or max memory*/
  if (curs != NULL)
    curs->c_close(curs);
  return no;
}



/*!
  \fn int get_db_obs_last(DB *dbp, struct obs_data *obs, int mode)
  \brief return the last (or fisrt) Obs in database. 
  \param dbp DB conection pointer already opened by caller
  \param obs pointer to a struct \ref obs_data where store result
  \param mode the mode of operation. If mode = 0 return first if mode = 1 return last

  It returns 1 if data is found, 0 otherwise
*/
int get_db_obs_last(DB *dbp, struct obs_data *obs, int mode)
{
  int no, ret;
  uint32_t flag;
  time_t ti;
  struct tm tim;
  DBC *curs;
  DBT key, data;

  /* set a cursor */
  dbp->cursor( dbp, NULL, &curs, 0);

  /* set initial key, search */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  no = 0;
  if (mode == 0)
    flag = DB_FIRST; 
  else
    flag = DB_LAST;

  if ((ret = curs->c_get(curs, &key, &data, flag)) == 0)
    {
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M", &tim);
      ti = timegm(&tim);
      obs[no].datime = ti;
      buf2obs(obs, data.data);
      no++;
    }

  /* close the cursor in case of end or max memory*/
  if (curs != NULL)
    curs->c_close(curs);
  return no;
}


/*!
  \fn int del_db_obs(DB *dbp, uint8_t mask, time_t tbegin, time_t tend)
  \brief Delete the data in a closed time interval with a mask
  \param dbp DB conection pointer already opened by caller
  \param mask with the matching data mask to be deleted
  \param tbegin Begin fo time interval
  \param tend end of time interval

  It returns the amount of deleted records
*/
int del_db_obs(DB *dbp, uint8_t mask, time_t tbegin, time_t tend)
{
  int no, ret;
  char datei[16], datef[16];
  uint32_t flag;
  time_t ti;
  struct tm tim;
  struct obs_data obs;
  DBC *curs;
  DBT key, data;

  /* set a cursor */
  dbp->cursor( dbp, NULL, &curs, 0);

  /* set first and last key from the interval*/
  memset(&tim, 0, sizeof(struct tm));
  ti = tbegin;
  gmtime_r(&ti, &tim);
  strftime(datei, 16, "%Y%m%d%H%M", &tim);
  ti = tend;
  gmtime_r(&ti, &tim);
  strftime(datef, 16, "%Y%m%d%H%M", &tim);

  /* set initial key, search */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  key.data = datei;
  key.size = strlen(datei) + 1;

  /* loop to search data */
  no = 0;
  flag = DB_SET_RANGE; /* flag for fisrt time */
  while ((ret = curs->c_get(curs, &key, &data, flag)) == 0)
    {
      /* first read */
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M", &tim);
      ti = timegm(&tim);
      if (tend == 0 || ti <= tend)
        {
          buf2obs( &obs, data.data);
          /* if mask matches the delete */
          /*if (mask == obs.mask)
            {*/
              if (curs->c_del(curs, 0) == 0)
                no++;
           /* }*/
        }
      else
        {
          /* we surpassed the end of interval */
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
    }

  /* close the cursor in case of end or max memory*/
  if (curs != NULL)
    curs->c_close(curs);
  return no;
}


#endif
