/* $Id: wwsr_utils.c 90 2009-05-31 22:00:39Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*!
  \file wwsr_utils.c
  \brief some util routines for wwsr
*/
#include "wwsreader.h"

/*!
  \fn void print_extreme(FILE *f, struct extrem_data *d, char *name, char *unit)
  \brief prints a exterme value readed from wwsr station
  \param f FILE pointer where to set the output
  \param d pointer to a struct \ref extrem_data when the extreme data are set
  \param name string with the name of parameter
  \param unit string with the unit used
*/
void print_extreme(FILE *f, struct extrem_data *d, char *name, char *unit)
{
  char aux[32];
  time_t ti;

  ti = timegm(&d->tim);
  if ( (TBEGIN == 0 || ti >= TBEGIN) &&
       (TEND == 0 || ti <= TEND))
    {
      strftime(aux, 32, "%Y-%m-%d %H:%M", &d->tim);
      fprintf(f, "%s:\t %6.1f %s \t%s\n", name, d->dat, unit, aux);
    }
  else
    {
      fprintf(f, gettext("%s:\t Out of selected date interval\n"), name);
    }
}


/*!
  \fn size_t read_station_buf(uint8_t *b, size_t limit)
  \brief Stores a binary image of station memory in BUFLOG
  \param b pointer to an array of uint8_t where to set the data
  \param limit limit of memory for \a b 

  returns the number of bytes readed
*/
size_t read_station_buf( uint8_t *b , size_t limit)
{
  int i, ni;
  unsigned int nob, buftemp, inipos, curpos, nr;
  uint16_t current;
  uint8_t bufob[32];
  char aux[16];
  struct obs_data obx;


  /*read main block (first 256 bytes) */
  printf(gettext("Reading main control block ... "));
  get_main_block((char *)b);
  printf("Done\n");

  /* current observation position in memory */
  current = read_uint16(b + 30);
  /*
     first read current data block, then it is replaced in memory because it may change when reading
     other data
  */
  if (current > 0xFFE0)
    curpos = 0xFFE0;
  else
    curpos = current;

  if (FROMFILE)
    {
      fseek(F, curpos, SEEK_SET);
      if ((nr = fread(bufob, 1, 32, F)) != 32)
        {
          fprintf(stderr, gettext("Problem reading data from file %s at position %u\n"), INPUTFILE, curpos);
        }
    }
  else
    {
      aux[1] = ( curpos >>8 & 0xFF ) ;
      aux[2] = curpos & 0xFF;
      aux[3] = ( curpos >>8 & 0xFF ) ;
      aux[4] = curpos & 0xFF;
      ni = 0; // Try up to 16 times
      do
        {
          _send_usb_msg("\xa1", aux+ 1, aux + 2, "\x20", "\xa1", aux + 3, aux + 4,"\x20");
          _read_usb_msg((char *)(bufob));
          ni++;
        }
      while ((buf2obs(&obx, &bufob[0]) || buf2obs(&obx, &(bufob[16]))) && ni <= 16);
      nr = 32;
    }


  /* read the rest of data */
  if (LASTN == 0)
    {
      printf(gettext("Reading all the data\n"));
      nob = 4080;
      for ( buftemp = 0x0100, i = 0; buftemp < 0x10000; buftemp += 32)
        {
          if (FROMFILE)
            {
              fseek(F, buftemp, SEEK_SET);
              if (fread(b + buftemp, 1, 32, F) != 32)
                {
                  fprintf(stderr, gettext("Problem reading data from file %s at position %u\n"), INPUTFILE, buftemp);
                  return buftemp;
                }
            }
          else
            {
              aux[1] = ( buftemp >>8 & 0xFF ) ;
              aux[2] = buftemp & 0xFF;
              aux[3] = ( buftemp >>8 & 0xFF ) ;
              aux[4] = buftemp & 0xFF;
              ni = 0; // Try up to 16 times
              do
              {
                _send_usb_msg("\xa1", aux+ 1, aux + 2, "\x20", "\xa1", aux + 3, aux + 4,"\x20");
                _read_usb_msg((char *)(b + buftemp));
                ni++;
              } 
              while ((buf2obs(&obx, b + buftemp) || buf2obs(&obx, b + buftemp + 16)) && ni <= 16);
            }
          i += 2;
          if ((i & 15) == 0)
            {
              fprintf(stdout, ".");
              fflush(stdout);
            }
          if ((i & 1023) == 0)
            {
              fprintf(stdout, "\n");
              fflush(stdout);
            }
        }
      /* Now restore current data block */
      memcpy(b + curpos, bufob, nr);
      return MEMSIZE;
    }
  else
    {
      /* current observation position in memory */
      current = read_uint16(b + 30);

      /* block of 32 bytes */
      inipos = current - 0x0100 - LASTN * 16;
      if (inipos & 0x0010)
        {
          nob = LASTN + 1;
        }
      else
        nob = LASTN;

      printf(gettext("Reading latest %d data\n"), LASTN);

      /* read the rest of data */
      for ( i = nob; i >= 0; i -= 2)
        {
          buftemp = get_pos (current, i);

          if (FROMFILE)
            {
              fseek(F, buftemp, SEEK_SET);
              if (fread(b + buftemp, 1, 32, F) != 32)
                {
                  fprintf(stderr, gettext("Problem reading data from file %s at position %u\n"), INPUTFILE, buftemp);
                  return buftemp;
                }
            }
          else
            {
              aux[1] = ( buftemp >>8 & 0xFF ) ;
              aux[2] = buftemp & 0xFF;
              aux[3] = ( buftemp >>8 & 0xFF ) ;
              aux[4] = buftemp & 0xFF;
              ni = 0; // Try up to 16 times
              do
              {
                _send_usb_msg("\xa1", aux+ 1, aux + 2, "\x20", "\xa1", aux + 3, aux + 4,"\x20");
                _read_usb_msg((char *)(b + buftemp));
                ni++;
              } 
              while (buf2obs(&obx, b + buftemp) && ni <= 16);
            }
        }

      /* Now restore current data block */
      memcpy(b + curpos, bufob, nr);
      return (256 + nob  * 32);
    }
}

/*!
  \fn size_t read_current(uint8_t *b, uint16_t current)
  \brief read the current (last value) in a file or in the wwsr
  \param b array of uint8_t where the data are set or the position of memory in wwsr
  \param current offset of b where to set the data
*/
size_t read_current(uint8_t *b, uint16_t current)
{
  char aux[16];

  if (FROMFILE)
    {
      fseek(F, current, SEEK_SET);
      if (fread(b + current, 1, 32, F) != 32)
        {
          fprintf(stderr, gettext("Problem reading data from file %s at position %u\n"), INPUTFILE, current);
          return 32;
        }
    }
  else
    {
      aux[1] = ( current >> 8 & 0xFF ) ;
      aux[2] = current & 0xFF;
      aux[3] = ( current >> 8 & 0xFF ) ;
      aux[4] = current & 0xFF;
      _send_usb_msg("\xa1", aux+ 1, aux + 2, "\x20", "\xa1", aux + 3, aux + 4,"\x20");
      _read_usb_msg((char *)(b + current));
      /* mark as current observation in mask bit */
      b[current + 15] &= 0x01;
    }
  return 32;
}




/*!
  \fn size_t get_main_block(char *buf)
  \brief get first 256 bytes, the main block
  \param buf buffer where to set the result
*/
size_t get_main_block(char *buf)
{

  /* case of reading from binary file */
  if (FROMFILE)
    {
      fseek(F, 0, SEEK_SET); /* move to the begining */
      return fread(buf, 1, 256, F);
    }

  /* get 256 bytes block and set them in buf2 */
  /* get first 32 bytes from first block */
  _send_usb_msg("\xa1","\x00","\x00","\x20","\xa1","\x00","\x00","\x20");
  _read_usb_msg(buf);
  if ( ( pdebug == 1) )
    {
      printf("000-031: ");
      print_bytes(buf, 32);
      printf("\n");
    }

  /* Next 32 bytes */
  _send_usb_msg("\xa1","\x00","\x20","\x20","\xa1","\x00","\x20","\x20");
  _read_usb_msg(buf + 32);
  if ( ( pdebug == 1) )
    {
      printf("032-063: ");
      print_bytes(buf + 32, 32);
      printf("\n");
    }

  /* Next 32 bytes */
  _send_usb_msg("\xa1","\x00","\x40","\x20","\xa1","\x00","\x40","\x20");
  _read_usb_msg(buf + 64);
  if ( ( pdebug == 1) )
    {
      printf("064-095: ");
      print_bytes(buf + 64, 32);
      printf("\n");
    }

  /* Next 32 bytes */
  _send_usb_msg("\xa1","\x00","\x60","\x20","\xa1","\x00","\x60","\x20");
  _read_usb_msg(buf + 96);
  if ( ( pdebug == 1) )
    {
      printf("096-123: ");
      print_bytes(buf + 96, 32);
      printf("\n");
    }

  /* Next 32 bytes */
  _send_usb_msg("\xa1","\x00","\x80","\x20","\xa1","\x00","\x80","\x20");
  _read_usb_msg(buf + 128);
  if ( ( pdebug == 1) )
    {
      printf("124-159: ");
      print_bytes(buf + 128, 32);
      printf("\n");
    }

  /* Next 32 bytes */
  _send_usb_msg("\xa1","\x00","\xa0","\x20","\xa1","\x00","\xa0","\x20");
  _read_usb_msg(buf + 160);
  if ( ( pdebug == 1) )
    {
      printf("160-191: ");
      print_bytes(buf + 160, 32);
      printf("\n");
    }

  /* Next 32 bytes */
  _send_usb_msg("\xa1","\x00","\xc0","\x20","\xa1","\x00","\xc0","\x20");
  _read_usb_msg(buf + 192);
  if ( ( pdebug == 1) )
    {
      printf("192-223: ");
      print_bytes(buf + 192, 32);
      printf("\n");
    }

  /* Latest 32 bytes */
  _send_usb_msg("\xa1","\x00","\xe0","\x20","\xa1","\x00","\xe0","\x20");
  _read_usb_msg(buf + 224);
  if ( ( pdebug == 1) )
    {
      printf("224-255: ");
      print_bytes(buf + 224, 32);
      printf("\n");
    }
  return 256;
}

/*!
  \fn void print_obs_verbose(FILE *f, struct obs_data *o, time_t tbegin, time_t tend, int viewmask)
  \brief print an array of struct \ref obs_data in a verbose manner
  \param f FILE pointer where to display data
  \param o pointer to the struct \ref obs_data to show
  \param tbegin low limit filter in time to display
  \param tend high limit in time to didplay
  \param viewmask int with bit parameter masks to print
*/
void print_obs_verbose(FILE *f, struct obs_data *o, time_t tbegin, time_t tend, int viewmask)
{
  struct tm tim;
  char aux[32];

  if ( (tbegin == 0 || o->datime >= tbegin) &&
       (tend == 0 || o->datime <= tend))
    {
      memset(&tim, 0, sizeof(struct tm));
      gmtime_r(&(o->datime), &tim);
      strftime(aux, 32, "%Y-%m-%d %H:%M UTC\n", &tim);
      fprintf(f, gettext("Date    : %s\n"), aux);
      fprintf(f, gettext("Interval: %u minutes\n"), (unsigned int) o->interval);
      if (viewmask & MASK_HUM_INT)
        fprintf(f, gettext("Hum int : %u %%\n"), (unsigned int) o->hum_int);
      if (viewmask & MASK_TEMP_INT)
        fprintf(f, gettext("Tem int : %.1f C\n"), (float)(o->temp_int) * 0.1);
      if (viewmask & MASK_HUM_EXT)
        fprintf(f, gettext("Hum ext : %u %%\n"), (unsigned int)o->hum_out);
      if (viewmask & MASK_TEMP_EXT)
        fprintf(f, gettext("Tem ext : %.1f C\n"), (float)(o->temp_out) * 0.1);
      if (viewmask & MASK_PRES)
        fprintf(f, gettext("Pressure: %.1f HPa\n"), (float)(o->press) * 0.1);
      if ((viewmask & MASK_WIND) == MASK_WIND)
        {
          fprintf(f, gettext("Wind    : %.1f m/s\n"), (float)(o->wspd) * 0.1);
          if (o->wdir > 16)
            o->wdir = 16;
          fprintf(f, gettext("Wdir    : %s \n"), wd[o->wdir]);
          fprintf(f, gettext("Gust    : %.1f m/s\n"), (float)(o->wgust) * 0.1);
        }
      if (viewmask & MASK_RAIN)
        fprintf(f, gettext("Ac. prec: %.1f mm\n"), (float)(o->prec) * 0.3);
      fprintf(f, "#-------------------\n");
    }
}

/*!
  \fn void print_obs(FILE *f, struct obs_data *o, char sep, time_t tbegin, time_t tend, int viewmask)
  \brief print an array of struct \ref obs_data in a verbose manner
  \param f FILE pointer where to display data
  \param o pointer to the struct \ref obs_data to show
  \param sep string to separate two items
  \param tbegin low limit filter in time to display
  \param tend high limit in time to didplay
  \param viewmask int with bit parameter masks to print
*/
void print_obs(FILE *f, struct obs_data *o, char *sep, time_t tbegin, time_t tend, int viewmask)
{
  struct tm tim;
  char aux[32], *c, lin[256];

  /* check if valid data */
  if ( o->mask == 0xFF || (o->hum_int == 0 && o->interval == 0))
    return;

  /* check about date */
  if ( (tbegin == 0 || o->datime >= tbegin) &&
       (tend == 0 || o->datime <= tend))
    {
      memset(&tim, 0, sizeof(struct tm));
      gmtime_r(&(o->datime), &tim);
      strftime(aux, 32, "%Y%m%d%H%M", &tim);
      c = lin + sprintf(lin, "%s%s", aux, sep);
      if (viewmask & MASK_INTERV)
        c += sprintf(c, "%u%s", (unsigned int) o->interval, sep);
      if (viewmask & MASK_HUM_INT)
        c += sprintf(c, "%.0lf%s", value((double) o->hum_int, find_cal_pol(&CALWWSR, o->datime,"hi")), sep);
      if (viewmask & MASK_TEMP_INT)
        c += sprintf(c, "%.1lf%s", value((double)(o->temp_int) * 0.1, find_cal_pol(&CALWWSR, o->datime,"ti")), sep);
      if (viewmask & MASK_HUM_EXT)
        c += sprintf(c, "%.0lf%s", value((double) o->hum_out, find_cal_pol(&CALWWSR, o->datime,"ho")), sep);
      if (viewmask & MASK_TEMP_EXT)
        c += sprintf(c, "%.1lf%s", value((double)(o->temp_out) * 0.1, find_cal_pol(&CALWWSR, o->datime,"to")), sep);
      if (viewmask & MASK_PRES)
        c += sprintf(c, "%.1lf%s", value((double)(o->press) * 0.1, find_cal_pol(&CALWWSR, o->datime,"p")), sep);
      if ((viewmask & MASK_WIND) == MASK_WIND)
        {
          c += sprintf(c, "%.1lf%s", value((double)(o->wspd) * 0.1, find_cal_pol(&CALWWSR, o->datime,"w")), sep);
          if (o->wdir > 16)
            o->wdir = 16;
          c += sprintf(c, "%s%s", wd[o->wdir], sep);
          c += sprintf(c, "%.1lf%s", value((double)(o->wgust) * 0.1, find_cal_pol(&CALWWSR, o->datime,"w")), sep);
        }
      if (viewmask & MASK_RAIN)
        c += sprintf(c, "%.1lf%s", value((double)(o->prec) * 0.3, find_cal_pol(&CALWWSR, o->datime,"r")), sep);
      *(c - 1) = '\n';
      fprintf( f, "%s", lin);
    }
}

/*!
  \fn void print_obs_nores(FILE *f, struct obs_data *o, char sep, int viewmask)
  \brief print an array of struct \ref obs_data in a verbose manner
  \param f FILE pointer where to display data
  \param o pointer to the struct \ref obs_data to show
  \param sep string to separate two items
  \param viewmask int with bit parameter masks to print
*/
void print_obs_nores(FILE *f, struct obs_data *o, char *sep, int viewmask)
{
  struct tm tim;
  char aux[32], *c, lin[256];
  struct cal_pol *b;

  /* check if valid data */
  if ( o->mask == 0xFF || (o->hum_int == 0 && o->interval == 0))
    return;

  memset(&tim, 0, sizeof(struct tm));
  gmtime_r(&(o->datime), &tim);
  strftime(aux, 32, "%Y%m%d%H%M", &tim);
  c = lin + sprintf(lin, "%s%s", aux, sep);
  if (viewmask & MASK_INTERV)
    c += sprintf(c, "%u%s", (unsigned int) o->interval, sep);
  if (viewmask & MASK_HUM_INT)
    c += sprintf(c, "%.0lf%s", value((double) o->hum_int, find_cal_pol(&CALWWSR, o->datime, "hi")), sep);
  if (viewmask & MASK_TEMP_INT)
    c += sprintf(c, "%.1lf%s", value((double)(o->temp_int) * 0.1, find_cal_pol(&CALWWSR, o->datime, "ti")), sep);
  if (viewmask & MASK_HUM_EXT)
    c += sprintf(c, "%.0lf%s", value((double) o->hum_out, find_cal_pol(&CALWWSR, o->datime, "ho")), sep);
  if (viewmask & MASK_TEMP_EXT)
    c += sprintf(c, "%.1lf%s", value((double)(o->temp_out) * 0.1, find_cal_pol(&CALWWSR, o->datime, "to")), sep);
  if (viewmask & MASK_PRES)
    c += sprintf(c, "%.1lf%s", value((double)(o->press) * 0.1, find_cal_pol(&CALWWSR, o->datime, "p")), sep);
  if ((viewmask & MASK_WIND) == MASK_WIND)
    {
      b = find_cal_pol(&CALWWSR, o->datime, "w");
      c += sprintf(c, "%.1lf%s", value((double)(o->wspd) * 0.1, b), sep);
      if (o->wdir > 16)
        o->wdir = 16;
      c += sprintf(c, "%s%s", wd[o->wdir], sep);
      c += sprintf(c, "%.1lf%s", value((double)(o->wgust) * 0.1, b), sep);
    }
  if (viewmask & MASK_RAIN)
    c += sprintf(c, "%.1lf%s", value((double)(o->prec) * 0.3, find_cal_pol(&CALWWSR, o->datime, "r")), sep);
  *(c - 1) = '\n';
  fprintf( f, "%s", lin);
}

/*!
  \fn void read_extremes(uint8_t *bf)
  \brief read the extremes of a main block image of wwsr memory
  \param bf array of uint8_t as an image of wwsr main block memory
*/
void read_extremes(uint8_t *bf)
{

  /* first the dates */
  read_date( &hin_max.tim, bf + 0x8d);
  read_date( &hin_min.tim, bf + 0x92);
  read_date( &hout_max.tim, bf + 0x97);
  read_date( &hout_min.tim, bf + 0x9c);
  read_date( &tin_max.tim, bf + 0xa1);
  read_date( &tin_min.tim, bf + 0xa6);
  read_date( &tout_max.tim, bf + 0xab);
  read_date( &tout_min.tim, bf + 0xb0);
  read_date( &chill_max.tim, bf + 0xb5);
  read_date( &chill_min.tim, bf + 0xba);
  read_date( &dwp_max.tim, bf + 0xbf);
  read_date( &dwp_min.tim, bf + 0xc4);
  read_date( &apres_max.tim, bf + 0xc9);
  read_date( &apres_min.tim, bf + 0xce);
  read_date( &rpres_max.tim, bf + 0xd3);
  read_date( &rpres_min.tim, bf + 0xd8);
  read_date( &wsp_max.tim, bf + 0xdd);
  read_date( &gust_max.tim, bf + 0xe2);
  read_date( &rh_max.tim, bf + 0xe7);
  read_date( &rd_max.tim, bf + 0xec);
  read_date( &rw_max.tim, bf + 0xf1);
  read_date( &rm_max.tim, bf + 0xf6);
  read_date( &rt_max.tim, bf + 0xfb);

  /* then the values */
  hin_max.dat = value((double) bf[0x62], find_cal_pol(&CALWWSR, mktime(&hin_max.tim), "hi"));
  hin_min.dat = value((double)bf[0x63], find_cal_pol(&CALWWSR, mktime(&hin_min.tim), "hi"));
  hout_max.dat = value((double)bf[0x64], find_cal_pol(&CALWWSR, mktime(&hout_max.tim), "ho"));
  hout_min.dat = value((double)bf[0x65], find_cal_pol(&CALWWSR, mktime(&hout_min.tim), "ho"));
  tin_max.dat = value((double) read_sint16(bf + 0x66) * 0.1, find_cal_pol(&CALWWSR, mktime(&tin_max.tim), "ti"));
  tin_min.dat = value((double) read_sint16(bf + 0x68) * 0.1, find_cal_pol(&CALWWSR, mktime(&tin_min.tim), "ti"));
  tout_max.dat = value((double) read_sint16(bf + 0x6a) * 0.1, find_cal_pol(&CALWWSR, mktime(&tout_max.tim), "to"));
  tout_min.dat = value((double) read_sint16(bf + 0x6c) * 0.1, find_cal_pol(&CALWWSR, mktime(&tout_min.tim), "to"));
  chill_max.dat = (double) read_sint16(bf + 0x6e) * 0.1;
  chill_min.dat = (double) read_sint16(bf + 0x70) * 0.1;
  dwp_max.dat = (double) read_sint16(bf + 0x72) * 0.1;
  dwp_min.dat = (double) read_sint16(bf + 0x74) * 0.1;
  apres_max.dat = value((double) read_uint16(bf + 0x76) * 0.1,find_cal_pol(&CALWWSR, mktime(&apres_max.tim), "p"));
  apres_min.dat = value((double) read_uint16(bf + 0x78) * 0.1,find_cal_pol(&CALWWSR, mktime(&apres_min.tim), "p"));
  rpres_max.dat = (double) read_uint16(bf + 0x7a) * 0.1;
  rpres_min.dat = (double) read_uint16(bf + 0x7c) * 0.1;
  wsp_max.dat = value((double) read_uint16(bf + 0x7e) * 0.1, find_cal_pol(&CALWWSR, mktime(&wsp_max.tim),"w"));
  gust_max.dat = value((double) read_uint16(bf + 0x80) * 0.1, find_cal_pol(&CALWWSR, mktime(&gust_max.tim),"w"));
  rh_max.dat = value((double) read_uint16(bf + 0x82) * 0.1, find_cal_pol(&CALWWSR, mktime(&rh_max.tim),"r"));
  rd_max.dat = value((double) read_uint16(bf + 0x84) * 0.1, find_cal_pol(&CALWWSR, mktime(&rd_max.tim),"r"));
  rw_max.dat = value((double) read_uint16(bf + 0x86) * 0.1, find_cal_pol(&CALWWSR, mktime(&rw_max.tim),"r"));
  rm_max.dat = value((double) read_uint16(bf + 0x88) * 0.1, find_cal_pol(&CALWWSR, mktime(&rm_max.tim),"r"));
  rt_max.dat = value((double) read_uint16(bf + 0x8a) * 0.1, find_cal_pol(&CALWWSR, mktime(&rt_max.tim),"r"));
}

/*!
  \fn void print_all_extremes(FILE *f, int viewmask)
  \brief print the extreme data 
  
*/
void print_all_extremes(FILE *f, int viewmask)
{
  if (viewmask & MASK_HUM_INT)
    {
      print_extreme(f, &hin_max,  gettext("Hum. int. max. "),"%");
      print_extreme(f, &hin_min,  gettext("Hum. int. min. "),"%");
    }
  if (viewmask & MASK_HUM_EXT)
    {
      print_extreme(f, &hout_max, gettext("Hum. out. max. "),"%");
      print_extreme(f, &hout_min, gettext("Hum. out. min. "),"%");
    }
  if (viewmask & MASK_TEMP_INT)
    {
      print_extreme(f, &tin_max,  gettext("Tem. int. max. "),"C");
      print_extreme(f, &tin_min,  gettext("Tem. int. min. "),"C");
    }
  if (viewmask & MASK_TEMP_EXT)
    {
      print_extreme(f, &tout_max, gettext("Tem. out. max. "),"C");
      print_extreme(f, &tout_min, gettext("Tem. out. min. "),"C");
    }
  if ((viewmask & MASK_TEMP_EXT) && (viewmask & MASK_WSPD))
    {
      print_extreme(f, &chill_max, gettext("Wnd. chill max."),"C");
      print_extreme(f, &chill_min, gettext("Wnd. chill min."),"C");
    }
  if ((viewmask & MASK_TEMP_EXT) && (viewmask & MASK_HUM_EXT))
    {
      print_extreme(f, &dwp_max, gettext( "Dew point max. "),"C");
      print_extreme(f, &dwp_min, gettext( "Dew point min. "),"C");
    }
  if (viewmask & MASK_PRES)
    {
      print_extreme(f, &apres_max, gettext("Abs. Pres. max."),"Hpa");
      print_extreme(f, &apres_min, gettext("Abs. Pres. min."),"Hpa");
      print_extreme(f, &rpres_max, gettext("Rel. Pres. max."),"Hpa");
      print_extreme(f, &rpres_min, gettext("Rel. Pres. min."),"Hpa");
    }
  if (viewmask & MASK_WSPD)
    {
      print_extreme(f, &wsp_max,  gettext("Wind speed max."),"m/s");
      print_extreme(f, &gust_max, gettext("Wind gust max. "),"m/s");
    }
  if (viewmask & MASK_RAIN)
    {
      print_extreme(f, &rh_max, gettext("Hour prec. max."), "mm");
      print_extreme(f, &rd_max, gettext("Day. prec. max."), "mm");
      print_extreme(f, &rw_max, gettext("Week prec. max."), "mm");
      print_extreme(f, &rm_max, gettext("Month prec. max"), "mm");
      print_extreme(f, &rt_max, gettext("Total prec. max"), "mm");
    }
}

